% SiSU 2.0

@title: #___#
 :subtitle: #___#

@creator:
 :author: #___#

@rights:
 :copyright: #___#
 :license: #___#

@classify:
 :topic_register: #___#
 :subject: #___#
 :type: #___#
 :loc: #___#
 :oclc: #___#
 :isbn: #___#

@date:
 :published: 20YY-MM-DD
 :added_to_site: 20YY-MM-DD
 :modified: 20YY-MM-DD

@make:
 :headings: none; none; none; none;
 :num_top: 1
 :breaks: new=:C; break=1
 :bold: / #___#/i
 :italics: / #___#/i
 :skin: #___#

@links:
 {SiSU}http://www.jus.uio.no/sisu/SiSU

:A~ @title @author

1~  #___#

