(add-to-list 'load-path (or (file-name-directory #$) (car load-path)))
(autoload 'sisu-mode "sisu-mode" "\
Major mode for editing SiSU markup files.
SiSU (http://www.sisudoc.org/) document structuring, publishing
and search.

\(fn)" t nil)
(add-to-list 'auto-mode-alist '("\\.sst\\'" . sisu-mode))
(add-to-list 'auto-mode-alist '("\\.ssm\\'" . sisu-mode))
(add-to-list 'auto-mode-alist '("\\.ssi\\'" . sisu-mode))
